;Simple DCF77 Clock for LED/VFD display
;AT90S2313 @ 4.194304 MHz (external XTAL)
;Fuses for ATTiny2313: lfuse=0xDD, hfuse=0xDB
;(change .elif to .elseif if your assembler complains, or use avra 1.3.0)
;
;VERSION HISTORY:
;  2011-06-24  Arne Rossius
;    * complete rewrite of old version 0.4 (2007-01-14) with new DCF77 receiver
;    * new options: - hour10 can use dp of other digit
;    *              - seconds
;    *              - AM/PM indicators
;    *              - seperate dp/indicator digit possible
;    *              - segment <=> portpin assignment
;    *              - date format (with weekday)
;  2011-06-29  Arne Rossius
;    * automatic setting of SEG_DDR, COM_DDR, SW_DDR and SW_PIN

;TODO: full H10-digit w/ zero display

.include "../include/2313def.inc"

;TIME AND DATE DISPLAY
.equ USE_SECONDS = 0 ;display seconds on digits 4 and 5
.equ TWELVE_HOURS = 0 ;use 12 hours display
.equ FLASHING_COLON = 0 ;colon flashing frequency (0=static, 1=1 Hz, 2=0.5 Hz)
;date format: see below (search for DATE01, DATE23, DATE45)

;DISPLAY PORT PINS (use value '8' for unused outputs, e.g. COM5 = 8)
.equ SEG_PORT = PORTB ;segment outputs for display
.equ SEG_INV = 1 ;invert segment outputs (1 = active low)
.equ SEG_A = 0
.equ SEG_B = 1
.equ SEG_C = 2
.equ SEG_D = 3
.equ SEG_E = 4
.equ SEG_F = 5
.equ SEG_G = 6
.equ SEG_DP = 7 ;set to 8 if unused
.equ ZERO_BLANKING = 1 ;blank 1st digit zero (0=never, 1=time, 2=date, 3=both)
.equ COM_PORT = PORTD ;common (digit) outputs for display
.equ COM_INV = 1 ;invert common outputs (1 = active low)
.equ COM0 = 3 ;digit 0 pin (hours 10s)
.equ COM1 = 4 ;digit 1 pin (hours 1s)
.equ COM2 = 5 ;digit 2 pin (minutes 10s)
.equ COM3 = 6 ;digit 3 pin (minutes 1s)
.equ COM4 = 8 ;digit 4 pin (sec10s or just colon/AM/PM)
.equ COM5 = 8 ;digit 5 pin (seconds 1s)
.equ COM_COLON = 1 ;digit with colon on it (valid values: 0 to 5)
.equ COM_AM = 1 ;digit with "AM" indicator on it
.equ COM_PM = 1 ;digit with "PM" indicator on it
.equ COM_H10 = 0 ;0   = use seperate digit (connected to COM0) for hour 10s
                 ;1~5 = use given digit's dp for hour 10s (COM0 unused)

;SWITCH PORT PINS
.equ SW_PORT = PORTD ;switch/push-button inputs
.equ SW_DATE = 0 ;"display date" button/switch
.equ SW_UTC = 1 ;"UTC time" switch
.equ DATE_TIMEOUT = 5 ;duration for date display (in seconds, max. 125)

;DCF77 SETTINGS
.equ DCF_INVERTED = 1 ;inverted DCF77 signal (1 = active-low pulses)
.equ DCF_PULLUP = 1 ;enable internal pullup for DCF77 input
.equ DCF_USE_INT0 = 1 ;DCF77 input: 0 = ICP1 (PD6), 1 = INT0 (PD2)

;DISPLAY SEGMENT PORT PINS
.equ DIGIT_COLON = 1<<SEG_DP
.equ DIGIT_AM = 0 ;set to 0 to disable AM indicator
.equ DIGIT_PM = 0 ;set to 0 to disable PM indicator
.equ DIGIT_0 = 1<<SEG_A | 1<<SEG_B | 1<<SEG_C | 1<<SEG_D | 1<<SEG_E | 1<<SEG_F
.equ DIGIT_1 = 1<<SEG_B | 1<<SEG_C
.equ DIGIT_2 = 1<<SEG_A | 1<<SEG_B | 1<<SEG_D | 1<<SEG_E | 1<<SEG_G
.equ DIGIT_3 = 1<<SEG_A | 1<<SEG_B | 1<<SEG_C | 1<<SEG_D | 1<<SEG_G
.equ DIGIT_4 = 1<<SEG_B | 1<<SEG_C | 1<<SEG_F | 1<<SEG_G
.equ DIGIT_5 = 1<<SEG_A | 1<<SEG_C | 1<<SEG_D | 1<<SEG_F | 1<<SEG_G
.equ DIGIT_6 = 1<<SEG_A | 1<<SEG_C | 1<<SEG_D | 1<<SEG_E | 1<<SEG_F | 1<<SEG_G
.equ DIGIT_7 = 1<<SEG_A | 1<<SEG_B | 1<<SEG_C
.equ DIGIT_8 = 1<<SEG_A | 1<<SEG_B | 1<<SEG_C | 1<<SEG_D | 1<<SEG_E | 1<<SEG_F | 1<<SEG_G
.equ DIGIT_9 = 1<<SEG_A | 1<<SEG_B | 1<<SEG_C | 1<<SEG_D | 1<<SEG_F | 1<<SEG_G
.equ DIGIT_L = 1<<SEG_D | 1<<SEG_E | 1<<SEG_F
.equ DIGIT_C = 1<<SEG_A | 1<<SEG_D | 1<<SEG_E | 1<<SEG_F
.equ DIGIT_P = 1<<SEG_A | 1<<SEG_B | 1<<SEG_E | 1<<SEG_F | 1<<SEG_G
.equ DIGIT_S = 1<<SEG_A | 1<<SEG_C | 1<<SEG_D | 1<<SEG_F | 1<<SEG_G
.equ DIGIT_d = 1<<SEG_B | 1<<SEG_C | 1<<SEG_D | 1<<SEG_E | 1<<SEG_G
.equ DIGIT_E = 1<<SEG_A | 1<<SEG_C | 1<<SEG_E | 1<<SEG_F | 1<<SEG_G
.equ DIGIT_DASH = 1<<SEG_G
.equ DIGIT_BLANK = 0
.equ DIGIT_ALL = 1<<SEG_A | 1<<SEG_B | 1<<SEG_C | 1<<SEG_D | 1<<SEG_E | 1<<SEG_F | 1<<SEG_G | 1<<SEG_DP

;===============================================================================

.def	lpmresult = R0
.def	seconds = R1
.def	delay2 = R2

.def	temp = R16
.def	temp2 = R17
.def	subsec = R18
.def	dcfbits = R19
.def	dcfflags = R20
.def	datecnt = R21
.def	display = R22
.def	disp_off = R23
.def	delay1 = R24
.def	flags = R25
	.equ	fHour10 = 0 ;set if hour 10 digit is '1'
	.equ	fAM = 1 ;set between 00:00 and 11:59
	.equ	fPM = 2 ;set between 12:00 and 23:59

;R29:R28 = Y pointer
;R31:R30 = Z pointer

;===============================================================================

.cseg
.org 0x000
	rjmp	start
	
.org OVF0addr
	rjmp	ovf0int

.equ DCF_USE_UTC = 1
.include "dcf77.asm"

;DATE FORMAT
.equ DATE01 = tMonth ;hours digits: tMonth, tWeekday, if COM_H10 = 0: tDay, tYear
.equ DATE23 = tDay ;minutes digits
.equ DATE45 = tWeekday ;seconds digits (only if USE_SECONDS = 1)

;===============================================================================

ovf0int:
	;timer 0 overflow interrupt (16 Hz)
	push	temp
	push	temp2
	push	YL
	in	temp, SREG
	push	temp
	
	;increment time
	dec	subsec
	brne	ovf0int_end
	ldi	subsec, 16
	cpi	datecnt, 0 ;decrement date timeout
	breq	PC+2
	dec	datecnt
	mov	temp, seconds
	rcall	bcdinc
	mov	seconds, temp
	cpi	temp, 0x60
	brlo	ovf0int_end
	clr	seconds
	rcall	inctime
	rcall	calculate_utc
	
ovf0int_end:
	;return
	pop	temp
	out	SREG, temp
	pop	YL
	pop	temp2
	pop	temp
	reti

;===============================================================================

;some checks

.if ((TWELVE_HOURS == 0) && (COM_H10 != 0))
	.error "COM_H10 must be set to 0 when not using 12-hour mode"
.endif

.if (((DATE01 == tDay) || (DATE01 == tYear)) && (TWELVE_HOURS != 0))
	.error "DATE01 must not be set to tYear or tDay when using 12-hour mode"
.endif

.if (FLASHING_COLON > 2)
	.error "FLASHING_COLON is out of range"
.endif

.if (ZERO_BLANKING > 3)
	.error "ZERO_BLANKING is out of range"
.endif

.if (DATE_TIMEOUT > 125)
	.error "DATE_TIMEOUT is out of range"
.endif

.if (USE_SECONDS != 0)
	.equ	MAX_COM = 5
.elif ((COM_COLON == 4) || (COM_AM == 4) || (COM_PM == 4))
	.equ	MAX_COM = 4
.else
	.equ	MAX_COM = 3
.endif
.if (COM_H10 == 0)
	.equ	MIN_COM = 0
.else
	.equ	MIN_COM = 1
.endif

.if ((COM_COLON < MIN_COM) || (COM_COLON > MAX_COM))
	.error "COM_COLON is out of range"
.endif
.if ((COM_H10 < MIN_COM) || (COM_H10 > MAX_COM))
	.error "COM_H10 is out of range"
.endif
.if ((COM_AM < MIN_COM) || (COM_AM > MAX_COM))
	.error "COM_AM is out of range"
.endif
.if ((COM_PM < MIN_COM) || (COM_PM > MAX_COM))
	.error "COM_PM is out of range"
.endif

.if ((MIN_COM > 0) && (COM0 < 8))
	.warning "COM0 unused, but not set to 8"
.endif
.if ((MAX_COM < 4) && (COM4 < 8))
	.warning "COM4 unused, but not set to 8"
.endif
.if ((MAX_COM < 5) && (COM5 < 8))
	.warning "COM5 unused, but not set to 8"
.endif

;===============================================================================

start:
	;set stack pointer
	ldi	temp, LOW(RAMEND)
	out	SPL, temp
	
	;disable analog comparator
	ldi	temp, 0x80
	out	ACSR, temp
	
	;init PORTB
.if (SEG_PORT == PORTB)
  .equ SEG_DDR = DDRB
	ldi	temp2, DIGIT_ALL
  .if (SEG_INV != 0)
	ldi	temp, DIGIT_ALL
  .else
	ldi	temp, 0x00
  .endif
.elif (COM_PORT == PORTB)
  .equ COM_DDR = DDRB
	ldi	temp2, 1<<COM0 | 1<<COM1 | 1<<COM2 | 1<<COM3 | 1<<COM4 | 1<<COM5
  .if (COM_INV != 0)
	ldi	temp, 1<<COM0 | 1<<COM1 | 1<<COM2 | 1<<COM3 | 1<<COM4 | 1<<COM5
  .else
	ldi	temp, 0x00
  .endif
.else
  .error "Neither segment nor common outputs use PORTB"
.endif
.if (SW_PORT == PORTB)
  .equ SW_DDR = DDRB
  .equ SW_PIN = PINB
	andi	temp2, ~(1<<SW_DATE | 1<<SW_UTC)
	ori	temp, 1<<SW_DATE | 1<<SW_UTC
.endif
	out	DDRB, temp2
	out	PORTB, temp
	
	;init PORTD
.if (SEG_PORT == PORTD)
  .equ SEG_DDR = DDRD
	ldi	temp2, DIGIT_ALL
  .if (SEG_INV != 0)
	ldi	temp, DIGIT_ALL
  .else
	ldi	temp, 0x00
  .endif
.elif (COM_PORT == PORTD)
  .equ COM_DDR = DDRD
	ldi	temp2, 1<<COM0 | 1<<COM1 | 1<<COM2 | 1<<COM3 | 1<<COM4 | 1<<COM5
  .if (COM_INV != 0)
	ldi	temp, 1<<COM0 | 1<<COM1 | 1<<COM2 | 1<<COM3 | 1<<COM4 | 1<<COM5
  .else
	ldi	temp, 0x00
  .endif
.else
  .error "Neither segment nor common outputs use PORTD"
.endif
.if (SW_PORT == PORTD)
  .equ SW_DDR = DDRD
  .equ SW_PIN = PIND
	andi	temp2, ~(1<<SW_DATE | 1<<SW_UTC)
	ori	temp, 1<<SW_DATE | 1<<SW_UTC
.endif
.if (DCF_USE_INT0 != 0)
	andi	temp2, ~0x04
  .if (DCF_PULLUP != 0)
	ori	temp, 0x04
  .else
	andi	temp, ~0x04
  .endif
.else
	andi	temp2, ~0x40
  .if (DCF_PULLUP != 0)
	ori	temp, 0x40
  .else
	andi	temp, ~0x40
  .endif
.endif
	out	DDRD, temp2
	out	PORTD, temp

	;init timer 0
	ldi	temp, 0x05 ;Clk/1024 = 16 Hz overflow interrupt
	out	TCCR0, temp
	ldi	temp, 1<<TOIE0
	out	TIMSK, temp
	
	;init registers
	ldi	display, 0
	ldi	datecnt, 0
	rcall	dcf_init
	
	;enable interrupts
	sei

;===============================================================================

loop:
	;poll date switch
.if (SW_DATE < 8)
	cpi	datecnt, 0
	brne	datesw_end
	sbis	SW_PIN, SW_DATE
	ldi	datecnt, DATE_TIMEOUT*2
datesw_end:
.endif

	;poll UTC switch
	ldi	YL, RAM_Time
.if (SW_UTC < 8)
	sbis	SW_PIN, SW_UTC
	ldi	YL, RAM_UTC
.endif
	
	;prepare data for next digit
	sbrs	dcfflags, dcfTimeOK
	rjmp	disp_dcfinfo
	cpi	datecnt, DATE_TIMEOUT+1
	brsh	disp_date
	cpi	display, 2
	brsh	disp_min
	;hours
	ldd	temp, Y+tHour
.if (TWELVE_HOURS != 0)
	andi	flags, ~(1<<fAM | 1<<fPM)
	ldi	temp2, 1<<fAM
	cpi	temp, 0x12 ;hour >= 12: PM
	brlo	PC+2
	ldi	temp2, 1<<fPM
	or	flags, temp2
	cpi	temp, 0x13 ;hour >= 13: subtract 12
	brlo	PC+4
	subi	temp, 0x12
	brhc	PC+2
	subi	temp, 0x06
	cpi	temp, 0x00 ;hour = 0: set hour to 12
	brne	PC+2
	ldi	temp, 0x12
.endif
.if (COM_H10 != 0)
	;prepare flag for leftmost digit
	andi	flags, ~(1<<fHour10)
	cpi	temp, 0x10
	brlo	PC+2
	ori	flags, 1<<fHour10
.endif
	rjmp	disp_encode
disp_min:
	cpi	display, 4
	brsh	disp_sec
	;minutes
	ldd	temp, Y+tMinute
	rjmp	disp_encode
disp_sec:
	;seconds or colon
.if (USE_SECONDS != 0)
	mov	temp, seconds
	rjmp	disp_encode
.else
	ldi	temp, DIGIT_BLANK
	rjmp	disp_end
.endif
	
disp_date:
	;display date
	andi	flags, ~(1<<fAM | 1<<fPM)
	cpi	display, 2
	brsh	disp_date2
	ldd	temp, Y+DATE01
.if (DATE01 == tWeekday)
	ori	temp, 0xF0 ;blank unused digit
.endif
.if (COM_H10 != 0)
	;prepare flag for leftmost digit
	andi	flags, ~(1<<fHour10)
	cpi	temp, 0x10
	brlo	PC+2
	ori	flags, 1<<fHour10
.endif
	rjmp	disp_encode
disp_date2:
	cpi	display, 4
	brsh	disp_date4
	ldd	temp, Y+DATE23
.if (DATE23 == tWeekday)
	ori	temp, 0xF0 ;blank unused digit
.endif
	rjmp	disp_encode
disp_date4:
	ldd	temp, Y+DATE45
.if (DATE45 == tWeekday)
	ori	temp, 0xF0 ;blank unused digit
.endif
	rjmp	disp_encode
	
disp_dcfinfo:
	;display DCF77 reception info
	andi	flags, ~(1<<fAM | 1<<fPM)
	cpi	display, 1
	brsh	disp_dcfinfo1
	;hour 10s: last received bit
.if (TWELVE_HOURS != 0)
	ldi	temp, DIGIT_BLANK
	cpi	dcfbits, 0
	brne	PC+2
	rjmp	disp_end
	sbrc	dcfflags, dcfStop
	rjmp	disp_end
	ldi	temp, 1<<SEG_C ;lower segment = 0
	lds	temp2, RAM_DCF2+5
	sbrc	temp2, 7
	ldi	temp, 1<<SEG_B ;upper segment = 1
.else
	ldi	temp, DIGIT_DASH
	cpi	dcfbits, 0
	brne	PC+2
	rjmp	disp_end
	sbrc	dcfflags, dcfStop
	rjmp	disp_end
	ldi	temp, DIGIT_0
	lds	temp2, RAM_DCF2+5
	sbrc	temp2, 7
	ldi	temp, DIGIT_1
.endif
	rjmp	disp_end
disp_dcfinfo1:
	cpi	display, 2
	brsh	disp_dcfinfo2
	;hour 1s: last error
.if (COM_H10 != 0)
	;prepare flag for leftmost digit (last received bit)
	andi	flags, ~(1<<fHour10)
	cpi	dcfbits, 0
	breq	disp_dcfinfo1_h10_end
	sbrc	dcfflags, dcfStop
	rjmp	disp_dcfinfo1_h10_end
	lds	temp2, RAM_DCF2+5
	sbrc	temp2, 7
	ori	flags, 1<<fHour10
disp_dcfinfo1_h10_end:
.endif
	ldi	temp, DIGIT_BLANK
	sbrc	dcfflags, dcfFirstTelegram
	ldi	temp, DIGIT_1 ;one telegram received sucessfully
	sbrc	dcfflags, dcfStop
	ldi	temp, DIGIT_DASH ;waiting (DCF stopped, but no error)
	mov	temp2, dcfflags
	andi	temp2, dcfErrMask
	cpi	temp2, dcfErrBitLength
	brne	PC+2
	ldi	temp, DIGIT_L ;Bit Length error
	cpi	temp2, dcfErrBitCount
	brne	PC+2
	ldi	temp, DIGIT_C ;Bit Count error
	cpi	temp2, dcfErrParity
	brne	PC+2
	ldi	temp, DIGIT_P ;Parity error
	cpi	temp2, dcfErrStartBit
	brne	PC+2
	ldi	temp, DIGIT_S ;Start Bit error
	cpi	temp2, dcfErrDST
	brne	PC+2
	ldi	temp, DIGIT_d ;invalid DST bits
	cpi	temp2, dcfErrPlausibility
	brne	PC+2
	ldi	temp, DIGIT_E ;Plausibility Check failed
	rjmp	disp_end
disp_dcfinfo2:
	cpi	display, 4
	brsh	disp_dcfinfo4
	;minutes: number of received bits
	mov	temp, dcfbits
	clr	temp2
disp_bin2bcd:
	cpi	temp, 10
	brlo	disp_bin2bcd_end
	subi	temp, 10
	subi	temp2, -0x10
	rjmp	disp_bin2bcd
disp_bin2bcd_end:
	add	temp, temp2
	rjmp	disp_encode
disp_dcfinfo4:
	;seconds/colon digit: display nothing
	ldi	temp, DIGIT_BLANK
	rjmp	disp_end

disp_encode:
	;convert to 7-segment
	sbrs	display, 0
	swap	temp
	andi	temp, 0x0F
	ldi	ZL, LOW(display_table*2)
	ldi	ZH, HIGH(display_table*2)
	add	ZL, temp
	ldi	temp, 0
	adc	ZH, temp
	lpm
	mov	temp, R0
	
	;zero blanking
.if (ZERO_BLANKING != 0)
	cpi	display, 0
	brne	disp_end
	cpi	temp, DIGIT_0
	brne	disp_end
  .if (ZERO_BLANKING == 1)
	cpi	datecnt, DATE_TIMEOUT+1
	brsh	disp_end
  .elif (ZERO_BLANKING == 2)
	cpi	datecnt, DATE_TIMEOUT+1
	brlo	disp_end
  .endif
	ldi	temp, DIGIT_BLANK
.endif
disp_end:
	
.if (COM_H10 != 0)
	;hour 10s is using dp of another digit
	cpi	display, COM_H10
	brne	h10_end
	sbrc	flags, fHour10
	ori	temp, 1<<SEG_DP
h10_end:
.endif

	;flashing colon
	cpi	display, COM_COLON
	brne	colon_end
	sbrc	dcfflags, dcfTimeOK
	rjmp	colon_regular
	;colon is DCF77 reception indicator
.if (DCF_USE_INT0 != 0)
	in	temp2, MCUCR
  .if (DCF_INVERTED == 0)
	sbrs	temp2, ISC00
  .else
	sbrc	temp2, ISC00
  .endif
.else
	in	temp2, TCCR1B
  .if (DCF_INVERTED == 0)
	sbrs	temp2, ICES1
  .else
	sbrc	temp2, ICES1
  .endif
.endif
	ori	temp, DIGIT_COLON
	rjmp	colon_end
colon_regular:
	;regular flashing colon
	cpi	datecnt, DATE_TIMEOUT+1 ;no colon when displaying date
	brsh	colon_end
.if (FLASHING_COLON == 1) ;1 Hz
	sbrs	subsec, 3
.elif (FLASHING_COLON == 2) ;0.5 Hz
	sbrs	seconds, 0
.endif ;else: static (always on)
	ori	temp, DIGIT_COLON
colon_end:

	;AM/PM indicators
	cpi	display, COM_AM
	brne	PC+3
	sbrc	flags, fAM
	ori	temp, DIGIT_AM
	cpi	display, COM_PM
	brne	PC+3
	sbrc	flags, fPM
	ori	temp, DIGIT_PM
	
	;prepare segment output data
	ldi	disp_off, DIGIT_BLANK
.if (SEG_INV != 0)
	com	temp
	com	disp_off
.endif
.if (SEG_PORT == SW_PORT)
	ori	temp, 1<<SW_DATE | 1<<SW_UTC
	ori	disp_off, 1<<SW_DATE | 1<<SW_UTC
.endif
.if (SEG_PORT == PORTD)
  .if (DCF_PULLUP != 0)
    .if (DCF_USE_INT0 != 0)
	ori	temp, 0x04
	ori	disp_off, 0x04
    .else
	ori	temp, 0x04
	ori	disp_off, 0x04
    .endif
  .else
    .if (DCF_USE_INT0 != 0)
	andi	temp, ~0x04
	andi	disp_off, ~0x04
    .else
	andi	temp, ~0x40
	andi	disp_off, ~0x40
    .endif
  .endif
.endif

	;prepare common outputs bitmask
	cpi	display, 0
	brne	PC+2
	ldi	temp2, 1<<COM0
	cpi	display, 1
	brne	PC+2
	ldi	temp2, 1<<COM1
	cpi	display, 2
	brne	PC+2
	ldi	temp2, 1<<COM2
	cpi	display, 3
	brne	PC+2
	ldi	temp2, 1<<COM3
	cpi	display, 4
	brne	PC+2
	ldi	temp2, 1<<COM4
	cpi	display, 5
	brne	PC+2
	ldi	temp2, 1<<COM5

	;prepare common output data
.if (COM_INV != 0)
	com	temp2
.endif
.if (COM_PORT == SW_PORT)
	ori	temp2, 1<<SW_DATE | 1<<SW_UTC
.endif
.if (COM_PORT == PORTD)
  .if (DCF_PULLUP != 0)
    .if (DCF_USE_INT0 != 0)
	ori	temp2, 0x04
    .else
	ori	temp2, 0x40
    .endif
  .else
    .if (DCF_USE_INT0 != 0)
	andi	temp2, ~0x04
    .else
	andi	temp2, ~0x40
    .endif
  .endif
.endif

	;select next digit
	inc	display
	cpi	display, MAX_COM + 1
	brlo	PC+2
	ldi	display, MIN_COM

	;delay (for 200 ~ 250 Hz Mux)
.if ((MAX_COM - MIN_COM) == 5)
	ldi	delay1, 4 ;6 active digits
.elif ((MAX_COM - MIN_COM) == 4)
	ldi	delay1, 5 ;5 active digits
.elif ((MAX_COM - MIN_COM) == 3)
	ldi	delay1, 6 ;4 active digits
.elif ((MAX_COM - MIN_COM) == 2)
	ldi	delay1, 8 ;3 active digits
.else
	.error "Number of active digits not between 3 and 6 (shouldn't happen)"
.endif
	clr	delay2
delay:
	dec	delay2
	brne	delay
	dec	delay1
	brne	delay

	;output new data
	out	SEG_PORT, disp_off
	rjmp	PC+1
	rjmp	PC+1
	out	COM_PORT, temp2
	rjmp	PC+1
	rjmp	PC+1
	out	SEG_PORT, temp
	
	rjmp	loop
	
;--------------------

display_table:
	;7-segment representations of 0x0 to 0xF (0xA to 0xF = blank)
	.db DIGIT_0, DIGIT_1, DIGIT_2, DIGIT_3
	.db DIGIT_4, DIGIT_5, DIGIT_6, DIGIT_7
	.db DIGIT_8, DIGIT_9, DIGIT_BLANK, DIGIT_BLANK
	.db DIGIT_BLANK, DIGIT_BLANK, DIGIT_BLANK, DIGIT_BLANK
